# logger.py
import logging
import locale
from datetime import datetime
from pathlib import Path
import re
from .config import DMT_LOG_TEMPLATE, DMT_LOG_TIME_FORMAT, DMT_LOG_LEVEL, DMT_LOG_PATH

# Define o locale para português do Brasil
try:
    locale.setlocale(locale.LC_TIME, 'pt_BR.utf8')
except locale.Error:
    print("Locale 'pt_BR.utf8' não disponível. Usando o padrão do sistema.")

class LoggerObj(logging.Logger):
    def __init__(self, name: str, log_path: str = DMT_LOG_PATH, log_level: int = DMT_LOG_LEVEL, 
                 log_template: str = DMT_LOG_TEMPLATE, time_format: str = DMT_LOG_TIME_FORMAT):
        """
        Classe Logger personalizada com suporte a templates dinâmicos.

        :param name: Nome do logger.
        :param log_path: Caminho do arquivo de log. Pode conter placeholders dinâmicos.
        :param log_level: Nível de log (INFO, DEBUG, etc.).
        :param log_template: Template de formatação do log.
        :param time_format: Formato da hora no log.
        """
        super().__init__(name, log_level)
        self.log_path = log_path.format(log_name=name)
        log_template = log_template.format(log_name=name)
        self.logging_path = self._setup_logger(log_template, time_format, log_level)

    def _setup_logger(self, log_format: str, time_format: str, log_level: int) -> str:
        """Configura o logger e define o arquivo de saída."""
        log_path = self._generate_log_path()

        # Verifica se o handler já foi adicionado para evitar duplicações
        if not any(isinstance(h, logging.FileHandler) and h.baseFilename == log_path for h in self.handlers):
            file_handler = logging.FileHandler(log_path, encoding="utf-8")
            file_handler.setLevel(log_level)
            file_handler.setFormatter(logging.Formatter(log_format, time_format))
            self.addHandler(file_handler)

        self.setLevel(log_level)
        self.propagate = False
        return log_path

    def _generate_log_path(self) -> str:
        """Gera o caminho correto do arquivo de log baseado no template e formatação dinâmica."""
        now = datetime.now()

        def replace_match(match):
            date_format = match.group(1)
            return now.strftime(date_format).replace(" ", "_")  # Evita espaços no caminho

        log_path = re.sub(r"\[now (.*?)\]", replace_match, self.log_path)

        # Cria o diretório do log se não existir
        log_folder = Path(log_path).parent
        log_folder.mkdir(parents=True, exist_ok=True)

        return log_path
